#include "Roue.h"
#include "Sommet.h"
#include "Util.h"
#include "SurfaceRectangulaire.h"
#include <algorithm>
#include <cmath>

Roue::Roue (const D3DXVECTOR3 &_origine, IDirect3DDevice9 *pDevice,
            const double Largeur, const double Rayon, const int NbSegments)
   : ObjetComplexe (_origine),
     m_Largeur (Largeur), m_Rayon (Rayon), m_NbSegments (NbSegments)
{
   const D3DXVECTOR3 Pos (0.0f, 0.0f, static_cast<float>(-m_Rayon));
   const double Alpha = 2 * D3DX_PI / (NbSegments);
   const double Theta = (D3DX_PI - Alpha) / 2;
   const double Hauteur = m_Rayon * std::sin(Theta);
   const double LongueurSurface =
      2.0 * std::sqrt (m_Rayon * m_Rayon - Hauteur * Hauteur);
   for (int i = 0; i < m_NbSegments; i++)
   {
      SurfaceRectangulaire *p =
         new SurfaceRectangulaire (Pos, pDevice,
                                   static_cast<float>(m_Largeur),
                                   static_cast<float>(LongueurSurface + LongueurSurface / m_NbSegments));
      D3DXMATRIX matRotation;
      ::D3DXMatrixRotationX (&matRotation, i * D3DX_PI * 2 / m_NbSegments);
      D3DXMATRIX matTranslation;
      ::D3DXMatrixTranslation (&matTranslation, GetPosition ().x, GetPosition ().y, GetPosition ().z);
      p->SetMatriceMonde (matRotation * matTranslation);
      m_ObjetsConstitutifs.push_back (p);
   }
}

D3DXVECTOR3 TournerVecteurX (const D3DXVECTOR3 &Vect, const float Angle)
{
   D3DXMATRIX mat;
   ::D3DXMatrixRotationX (&mat, Angle);
   D3DXVECTOR4 VectResult;
   D3DXVec3Transform (&VectResult, &Vect, &mat);
   return D3DXVECTOR3 (VectResult.x, VectResult.y, VectResult.z) ;
}

Roue::Roue (const D3DXVECTOR3 &_origine, IDirect3DDevice9 *pDevice,
            const double Largeur, const double Rayon, const int NbSegments,
            const D3DMATERIAL9 &Materiau)
   : ObjetComplexe (_origine),
     m_Largeur (Largeur), m_Rayon (Rayon), m_NbSegments (NbSegments)
{
   const D3DXVECTOR3 Pos (0.0f, 0.0f, static_cast<float>(-m_Rayon));
   const double Alpha = 2 * D3DX_PI / (NbSegments);
   const double Theta = (D3DX_PI - Alpha) / 2;
   const double Hauteur = m_Rayon * std::sin(Theta);
   const double LongueurSurface =
      2.0 * std::sqrt (m_Rayon * m_Rayon - Hauteur * Hauteur);
   const double InclinaisonNormale = 2 * D3DX_PI - 2 * Theta;
   std::vector<D3DXVECTOR3> v;
   const D3DXVECTOR3 NormaleDebut = TournerVecteurX (D3DXVECTOR3 (0.0f, 0.0f, 1.0f), -static_cast<const float>(InclinaisonNormale));
   const D3DXVECTOR3 NormaleFin   = TournerVecteurX (D3DXVECTOR3 (0.0f, 0.0f, 1.0f), static_cast<const float>(InclinaisonNormale));
   v.push_back (NormaleDebut);
   v.push_back (NormaleDebut);
   v.push_back (NormaleFin);
   v.push_back (NormaleFin);
   for (int i = 0; i < m_NbSegments; i++)
   {
      SurfaceRectangulaire *p =
         new SurfaceRectangulaire (Pos, pDevice,
                                   static_cast<float>(m_Largeur),
                                   static_cast<float>(LongueurSurface + LongueurSurface / m_NbSegments),
                                   Materiau,
                                   v);
      D3DXMATRIX matRotation;
      ::D3DXMatrixRotationX (&matRotation, i * D3DX_PI * 2 / m_NbSegments);
      D3DXMATRIX matTranslation;
      ::D3DXMatrixTranslation (&matTranslation, GetPosition ().x, GetPosition ().y, GetPosition ().z);
      p->SetMatriceMonde (matRotation * matTranslation);
      m_ObjetsConstitutifs.push_back (p);
   }
}
namespace
{
   class Dessiner
   {
      IDirect3DDevice9 *m_pDevice;
      const D3DXMATRIX m_Contexte;
   public:
      Dessiner (IDirect3DDevice9 *pDevice,
                const D3DXMATRIX &Contexte)
         : m_pDevice (pDevice), m_Contexte (Contexte) { }
      void operator() (IDessinable *p)
      {
         D3DXMATRIX Temp = p->GetMatriceMonde () * m_Contexte;
         m_pDevice->SetTransform (D3DTS_WORLD, &Temp);
         p->DrawNoTransform (m_pDevice);
         m_pDevice->SetTransform (D3DTS_WORLD, &m_Contexte);
      }
   };
}

void Roue::DrawNoTransform (IDirect3DDevice9 *pDevice)
{
   std::for_each (m_ObjetsConstitutifs.begin (), m_ObjetsConstitutifs.end (), Dessiner (pDevice, GetMatriceMonde ()));
}

