#include "SurfaceRectangulaire.h"

const float
   SurfaceRectangulaire::HAUTEUR = 5.0f,
   SurfaceRectangulaire::LARGEUR = 5.0f;

SurfaceRectangulaire::__MateriauDefaut SurfaceRectangulaire::_MateriauDefaut;

SurfaceRectangulaire::__MateriauDefaut::__MateriauDefaut ()
{
   ::memset (&m_Materiau, 0, sizeof (m_Materiau));
   // Initialiser le RGBA pour la rflection ambiante
   m_Materiau.Ambient.r = 0.25f;
   m_Materiau.Ambient.g = 0.75f;
   m_Materiau.Ambient.b = 0.25f;
   m_Materiau.Ambient.a = 1.0f;
   // Initialiser le RGBA pour la rflexion diffuse
   m_Materiau.Diffuse.r = 0.5f;
   m_Materiau.Diffuse.g = 0.0f;
   m_Materiau.Diffuse.b = 0.5f;
   m_Materiau.Diffuse.a = 1.0f;
   // Initialiser la couleur et l'intensit de la spcularit (lustre)
   m_Materiau.Specular.r = 1.0f;
   m_Materiau.Specular.g = 1.0f;
   m_Materiau.Specular.b = 1.0f;
   m_Materiau.Specular.a = 1.0f;
   // Initialiser le RGBA pour la couleur missive
   m_Materiau.Emissive.r = 0.0f;
   m_Materiau.Emissive.g = 0.0f;
   m_Materiau.Emissive.b = 0.0f;
   m_Materiau.Emissive.a = 0.0f;
}


SurfaceRectangulaire::SurfaceRectangulaire
   (const D3DXVECTOR3 &_origine, IDirect3DDevice9 *pDevice,
    const float Largeur, const float Hauteur,
    const D3DMATERIAL9 &Materiau,
    const std::vector<D3DXVECTOR3> &_Normales)
   : IDessinable (_origine),
     m_Largeur (Largeur), m_Hauteur (Hauteur),
     m_pIB (0), m_pVB (0), m_Materiau (Materiau)
{
   // valuer les normales
   std::vector<D3DXVECTOR3> Normales;
   const D3DXVECTOR3 NORMALE_DEFAUT = D3DXVECTOR3 ( 0.0f,  0.0f,  1.0f ); // dessus
   Normales.push_back (_Normales.size () >= 1? _Normales[0] : NORMALE_DEFAUT);
   Normales.push_back (_Normales.size () >= 2? _Normales[1] : NORMALE_DEFAUT);
   Normales.push_back (_Normales.size () >= 3? _Normales[2] : NORMALE_DEFAUT);
   Normales.push_back (_Normales.size () >= 4? _Normales[3] : NORMALE_DEFAUT);
   SOMMET *Sommets;
   pDevice->CreateVertexBuffer ( NB_SOMMETS_DISTINCTS * sizeof (SOMMET),
                                 0,
                                 SOMMET::D3DFVF_SOMMET,
                                 D3DPOOL_DEFAULT,
                                 &m_pVB,
                                 0);
   m_pVB->Lock (0, 0, reinterpret_cast<void **>(&Sommets), 0);
   Sommets[0] = SOMMET (D3DXVECTOR3 (-m_Largeur/2, -m_Hauteur/2, 0.0f) + _origine,  Normales[0]);
   Sommets[1] = SOMMET (D3DXVECTOR3 (-m_Largeur/2,  m_Hauteur/2, 0.0f) + _origine,  Normales[1]);
   Sommets[2] = SOMMET (D3DXVECTOR3 ( m_Largeur/2,  m_Hauteur/2, 0.0f) + _origine,  Normales[2]);
   Sommets[3] = SOMMET (D3DXVECTOR3 ( m_Largeur/2, -m_Hauteur/2, 0.0f) + _origine,  Normales[3]);
   Sommets[4] = SOMMET (D3DXVECTOR3 (-m_Largeur/2, -m_Hauteur/2, 0.0f) + _origine, -Normales[0]);
   Sommets[5] = SOMMET (D3DXVECTOR3 (-m_Largeur/2,  m_Hauteur/2, 0.0f) + _origine, -Normales[1]);
   Sommets[6] = SOMMET (D3DXVECTOR3 ( m_Largeur/2,  m_Hauteur/2, 0.0f) + _origine, -Normales[2]);
   Sommets[7] = SOMMET (D3DXVECTOR3 ( m_Largeur/2, -m_Hauteur/2, 0.0f) + _origine, -Normales[3]);
   m_pVB -> Unlock ();
   InitIndexBuffer (pDevice);
}

void SurfaceRectangulaire::InitIndexBuffer (IDirect3DDevice9 *pDevice)
{
   const int NB_INDEX = NB_FACES * NB_PRIMITIVES_FACE * NB_SOMMETS_PRIMITIVE;
   const WORD SourceIndex[NB_INDEX] =
   {
      0, 1, 2,
      0, 2, 3,
      2, 1, 0,
      3, 2, 0
   };
   WORD *pIndex;
   pDevice ->CreateIndexBuffer(NB_INDEX * sizeof (WORD),
                               D3DUSAGE_WRITEONLY,
                               D3DFMT_INDEX16,
                               D3DPOOL_DEFAULT,
                               &m_pIB,
                               0);
   m_pIB->Lock (0, 0, reinterpret_cast<void **> (&pIndex), 0);
   std::copy (&SourceIndex[0], &SourceIndex[NB_INDEX], pIndex);
   m_pIB->Unlock ();
}

void SurfaceRectangulaire::DrawNoTransform (IDirect3DDevice9 *pDevice)
{
   const int NB_INDEX = NB_FACES * NB_PRIMITIVES_FACE * NB_SOMMETS_PRIMITIVE;
   pDevice->SetMaterial (&m_Materiau);
   pDevice->SetStreamSource (0, m_pVB, 0, sizeof (SOMMET));
   pDevice->SetFVF (SOMMET::D3DFVF_SOMMET);
   pDevice->SetIndices (m_pIB);
   pDevice->DrawIndexedPrimitive (D3DPT_TRIANGLELIST, 0, 0, NB_INDEX, 0, NB_PRIMITIVES);
}

SurfaceRectangulaire::~SurfaceRectangulaire()
{
   Relacher (m_pVB);
   Relacher (m_pIB);
}